
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <cmqc.h>
#include <cmqstrc.h>

#ifndef RoundTo4
#define RoundTo4(u) (((u)+ (( 4-((u)%4))%4)))
#endif

int main(int argc, char **argv)
{
  MQOD     od = {MQOD_DEFAULT};    
  MQMD     md = {MQMD_DEFAULT};    
  MQPMO    pmo = {MQPMO_DEFAULT};   
  MQRFH2   rfh2Header = {MQRFH2_DEFAULT};

  MQHCONN  hConn;                   
  MQHOBJ   hObj;                   
  MQLONG   openOptions;              
  MQLONG   CC;               
  MQLONG   RC;    

  MQLONG   messageLen;                
  MQLONG   folderLen; // this must be a 4-byte integer

  char     qMgrName[50];             
  char     buffer[65535] = {0};     
  char     inputLine[BUFSIZ] = {0};

  int   offset;
  char *body;
  char *folder;
  int   s;

  if (argc < 2)
  {
    printf("Required parameter missing - queue name\n");
    exit(1);
  }

  qMgrName[0] = 0;  
  if (argc > 2) 
  {
    strncpy(qMgrName, argv[2], (size_t)MQ_Q_MGR_NAME_LENGTH);
  }

  MQCONN(qMgrName, &hConn, &CC, &RC);         
  if (CC == MQCC_FAILED)
  {
    printf("MQCONNX ended with RC code %d\n", RC);
    exit((int)RC );
  }

  strncpy(od.ObjectName, argv[1], (size_t)MQ_Q_NAME_LENGTH);
  printf("target queue is %s\n", od.ObjectName);

  openOptions = MQOO_OUTPUT | MQOO_FAIL_IF_QUIESCING;                        
  MQOPEN(hConn, &od, openOptions, &hObj, &CC, &RC);               
  if (RC != MQRC_NONE)
  {
    printf("MQOPEN ended with RC code %d\n", RC);
    exit(1);
  }

  printf("Enter folder strings (empty string ends):\n");
  offset = MQRFH_STRUC_LENGTH_FIXED_2; // 36

  do 
  {
    memset(inputLine,0,sizeof(inputLine));
    folder = fgets(inputLine,sizeof(inputLine)-1,stdin);
    // Remove trailing LF
    if (folder[strlen(folder)-1] == '\n')  
    {
       folder[strlen(folder) -1] = 0;
    }

    folderLen = RoundTo4(strlen(folder)); // Length must be multiple of 4. 
    if (folderLen) 
    {
      // NameValueLength - how long is the adjacent folder
      memcpy(&buffer[offset],&folderLen,sizeof(folderLen));
      offset += sizeof(folderLen);

      // NameValueData - copy in the actual length read from input, not the rounded value.
      // But bump the offset by the rounded amount, ready for next element. Gaps caused by 
      // rounding up are ignored.
      memcpy(&buffer[offset],folder,strlen(folder)); 
      offset += folderLen; 
    }
  } while(folderLen);

  // Fill out the RFH2 header block
  memcpy(rfh2Header.Format, MQFMT_STRING,(size_t)MQ_FORMAT_LENGTH); // Message body will be a string
  rfh2Header.CodedCharSetId = MQCCSI_INHERIT;
  rfh2Header.StrucLength = offset; // We now know the complete length, including the fixed header 
  rfh2Header.NameValueCCSID = 1208L;
  memcpy(buffer,&rfh2Header,sizeof(rfh2Header)); // Copy the header into the start of the buffer now we know lengths 
  
  // And tell the MQMD that there is an RFH2
  memcpy(md.Format,  MQFMT_RF_HEADER_2, (size_t)MQ_FORMAT_LENGTH);

  // Message body is a single input string
  printf("Enter body string:\n");
  memset(inputLine,0,sizeof(inputLine));

  body = fgets(inputLine,sizeof(inputLine)-1,stdin);
  s = strlen(body);
  if (body[s-1] == '\n') 
  {
     body[s-1] = 0;
     s--;
  }

  memcpy(&buffer[offset],body,s);
  messageLen = offset + s;

  pmo.Options = MQPMO_NO_SYNCPOINT | MQPMO_FAIL_IF_QUIESCING;
  MQPUT(hConn, hObj, &md, &pmo, messageLen, buffer, &CC,&RC);       
  if (RC != MQRC_NONE)
  {
    printf("MQPUT ended with RC code %s [%d]\n", MQRC_STR(RC),RC);
  }
     
  MQCLOSE(hConn, &hObj, 0, &CC, &RC);  
  MQDISC(&hConn, &CC, &RC);    

  return(0);
}
